<?php

namespace Drupal\block_entity_form\Plugin\Block;

use Drupal\block_entity_form\EntityFormService;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\user\EntityOwnerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block for creating a new content entity.
 *
 * @Block(
 *   id = "block_entity_form",
 *   admin_label = @Translation("Create Entity Form"),
 *   category = @Translation("Create Entity Form"),
 * )
 */
class BlockEntityFormBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  protected EntityFormBuilderInterface $entityFormBuilder;

  /**
   * @var \Drupal\block_entity_form\EntityFormService
   */
  private EntityFormService $entityFormService;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityFormBuilderInterface $entity_form_builder, EntityFormService $entityFormService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFormBuilder = $entity_form_builder;
    $this->entityFormService = $entityFormService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity.form_builder'),
      $container->get('block_entity_form.service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'entity_type' => '',
      'bundle' => '',
      'form_mode' => 'default',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account): AccessResultInterface {
    if ($this->configuration['entity_type'] === 'user' && $account->isAnonymous()) {
      $registrationUrl = Url::fromRoute('user.register');
      return $registrationUrl->access($account, TRUE);
    }
    $access = $this->entityTypeManager
      ->getAccessControlHandler($this->configuration['entity_type'])
      ->createAccess($this->configuration['bundle'], $account, [], TRUE);
    return $access;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $form['#prefix'] = '<div id="create-entity-block-form">';
    $form['#suffix'] = '</div>';
    $form['entity_type_bundle'] = [
      '#title' => $this->t('Entity Type + Bundle'),
      '#type' => 'select',
      '#options' => $this->entityFormService->getContentTypesOptions(),
      '#required' => TRUE,
      '#default_value' => $this->configuration['entity_type'] . '.' . $this->configuration['bundle'],
      "#ajax" => [
        'callback' => [$this, 'updateFormModes'],
        'event' => 'change',
        'progress' => [
          'type' => 'throbber',
          'message' => t('Loading Form Modes...'),
        ],
      ],
    ];
    $form['form_mode'] = [
      "#prefix" => '<div id="entity-form-mode">',
      "#suffix" => '</div>',
      '#type' => 'select',
      '#options' => $this->entityFormService->getFormModeOptions($this->configuration['entity_type']),
      "#validated" => TRUE,
      '#title' => $this->t('Form mode'),
      '#default_value' => $this->configuration['form_mode'],
    ];
    return $form;
  }

  /**
   * Updates the form modes for the selected entity
   *
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public function updateFormModes($form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    $selected_entity_type_bundle = $form_values['settings']['entity_type_bundle'];
    $values = explode('.', $selected_entity_type_bundle);
    $options = $this->entityFormService->getFormModeOptions($values[0]);
    $form['settings']['form_mode']['#options'] = $options;
    $response = new AjaxResponse();
    $replaceCommand = new ReplaceCommand('#entity-form-mode', $form['settings']['form_mode']);
    $response->addCommand($replaceCommand);
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $selected_entity_type_bundle = $form_state->getValue('entity_type_bundle');
    $values = explode('.', $selected_entity_type_bundle);
    $this->configuration['entity_type'] = $values[0];
    $this->configuration['bundle'] = $values[1];
    $this->configuration['form_mode'] = $form_state->getValue('form_mode');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $values = [];
    if ($this->entityTypeManager->getDefinition($this->configuration['entity_type'])
      ->hasKey('bundle')){
      $bundleKey = $this->entityTypeManager
        ->getDefinition($this->configuration['entity_type'])
        ->getKey('bundle');
      $values = [$bundleKey => $this->configuration['bundle']];
    }
    $entity = $this->entityTypeManager
      ->getStorage($this->configuration['entity_type'])
      ->create($values);
    if ($entity instanceof EntityOwnerInterface) {
      $entity->setOwnerId(\Drupal::currentUser()->id());
    }
    $entity_type_definition = $this->entityTypeManager
      ->getDefinition($this->configuration['entity_type'], TRUE);
    $form_handlers = $entity_type_definition->getHandlerClasses()['form'];
    if (empty($form_handlers[$this->configuration['form_mode']])) {
      if(isset($form_handlers['default'])){
        $validOperation = 'default';
      }
      elseif(isset($form_handlers['add'])){
        $validOperation = 'add';
      }
      elseif(isset($form_handlers['edit'])){
        $validOperation = 'edit';
      }
      else{
        return [];
      }
      $entity_type_definition->setFormClass($this->configuration['form_mode'], $form_handlers[$validOperation]);
    }
    $form = $this->entityFormBuilder->getForm($entity, $this->configuration['form_mode']);
    return $form;
  }

}
