<?php

namespace Drupal\block_entity_form\Plugin\views\area;

use Drupal\block_entity_form\EntityFormService;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\views\Plugin\views\area\AreaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an area handler which renders an entity in a certain view mode.
 *
 * @ingroup views_area_handlers
 *
 * @ViewsArea("views_block_entity_form_area")
 */
class BlockEntityFormArea extends AreaPluginBase implements ContainerFactoryPluginInterface {

  const PLUGIN_ID = 'block_entity_form';

  /**
   * The block manager service.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected BlockManagerInterface $blockManager;

  /**
   * @var \Drupal\block_entity_form\EntityFormService
   */
  private EntityFormService $entityFormService;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, BlockManagerInterface $block_manager, EntityFormService $entityFormService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->blockManager = $block_manager;
    $this->entityFormService = $entityFormService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.block'),
      $container->get('block_entity_form.service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['entity_type_bundle'] = ['default' => ''];
    $options['form_mode'] = ['default' => ''];
    $options['block_title'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   *
   * Renders the custom block using its configuration.
   */
  public function render($empty = FALSE) {
    // Get the stored configuration from this Views area plugin.
    $entityInfo = explode('.', $this->options['entity_type_bundle']);
    $block_config = [
      'entity_type' => $entityInfo[0],
      'bundle' => $entityInfo[1],
      'form_mode' => $this->options['form_mode'],
    ];
    /** @var \Drupal\block_entity_form\Plugin\Block\BlockEntityFormBlock $blockInstance */
    $blockInstance = $this->blockManager->createInstance(self::PLUGIN_ID, $block_config);
    $access = $blockInstance->access(\Drupal::currentUser());
    if (!$access) {
      return [];
    }
    $build = $blockInstance->build();
    $build['block_title'] = [
      '#markup' => $this->options['block_title'],
      '#weight' => -100,
    ];
    if (method_exists($blockInstance, 'getCache')) {
      $build['#cache'] = $blockInstance->getCache();
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   *
   * Build the configuration form for this Views area plugin.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['block_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Block Title'),
      '#default_value' => $this->options['block_title'],
    ];
    $form['entity_type_bundle'] = [
      '#title' => $this->t('Entity Type + Bundle'),
      '#type' => 'select',
      '#options' => $this->entityFormService->getContentTypesOptions(),
      '#required' => TRUE,
      '#default_value' => $this->options['entity_type_bundle'],
      '#ajax' => [
        'callback' => __CLASS__ .'::_block_entity_form_ajax_update_form_modes',
        'event' => 'change',
      ],
    ];
    $entityInfo = explode('.', $this->options['entity_type_bundle']);
    $form['form_mode'] = [
      "#prefix" => '<div id="entity-form-mode">',
      "#suffix" => '</div>',
      '#type' => 'select',
      '#options' => $this->entityFormService->getFormModeOptions($entityInfo[0]),
      "#validated" => TRUE,
      '#title' => $this->t('Form mode'),
      '#default_value' => $this->options['form_mode'],
    ];
  }

  /**
   * Callback to update form modes for selected entity type
   *
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   */
  public static function _block_entity_form_ajax_update_form_modes($form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();
    $selected_entity_type_bundle = $form_values['options']['entity_type_bundle'];
    $values = explode('.', $selected_entity_type_bundle);
    $options = \Drupal::service('block_entity_form.service')
      ->getFormModeOptions($values[0]);
    $form['options']['form_mode']['#options'] = $options;
    $response = new AjaxResponse();
    $replaceCommand = new ReplaceCommand('#entity-form-mode', $form['options']['form_mode']);
    $response->addCommand($replaceCommand);
    return $response;
  }

}
